# Generated by Django 4.2.7 on 2023-11-22 12:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    operations = [
        migrations.CreateModel(
            name='SignupCode',
            fields=[
                ('code', models.CharField(editable=False, max_length=40, primary_key=True, serialize=False, verbose_name='code')),
                ('ipaddr', models.GenericIPAddressField(verbose_name='ip address')),
                ('link', models.URLField(blank=True, default='', help_text='Contains a link to some source, will be supplemented with a request parameter with an instance code inside email.', verbose_name='email link')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'signup code',
                'verbose_name_plural': 'signup codes',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PasswordResetCode',
            fields=[
                ('code', models.CharField(editable=False, max_length=40, primary_key=True, serialize=False, verbose_name='code')),
                ('ipaddr', models.GenericIPAddressField(verbose_name='ip address')),
                ('link', models.URLField(blank=True, default='', help_text='Contains a link to some source, will be supplemented with a request parameter with an instance code inside email.', verbose_name='email link')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'password reset code',
                'verbose_name_plural': 'password reset codes',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EmailChangeCode',
            fields=[
                ('code', models.CharField(editable=False, max_length=40, primary_key=True, serialize=False, verbose_name='code')),
                ('ipaddr', models.GenericIPAddressField(verbose_name='ip address')),
                ('link', models.URLField(blank=True, default='', help_text='Contains a link to some source, will be supplemented with a request parameter with an instance code inside email.', verbose_name='email link')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('new_email', models.EmailField(help_text='New email address will be related with user after email change verification.', max_length=255, verbose_name='new email address')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'email change code',
                'verbose_name_plural': 'email change codes',
                'abstract': False,
            },
        ),
    ]
