<!DOCTYPE html>
<html lang="en-US">
  <head>
    <meta charset="UTF-8" />
    <title>Signup verification</title>
  </head>
  <body>
    <p>
      To verify your account click
      <a href="{{ link }}?code={{ code }}">here</a>.
    </p>
  </body>
</html>
